@@ -32,16 +32,16 @@ public:
   virtual ~SplashFTFont();
 
   // Munge xFrac and yFrac before calling SplashFont::getGlyph.
-  virtual GBool getGlyph(int c, int xFrac, int yFrac,
+  virtual GBool getGlyph(Guint c, int xFrac, int yFrac,
 			 SplashGlyphBitmap *bitmap);
 
   // Rasterize a glyph.  The <xFrac> and <yFrac> values are the same
   // as described for getGlyph.
-  virtual GBool makeGlyph(int c, int xFrac, int yFrac,
+  virtual GBool makeGlyph(Guint c, int xFrac, int yFrac,
 			  SplashGlyphBitmap *bitmap);
 
   // Return the path for a glyph.
-  virtual SplashPath *getGlyphPath(int c);
+  virtual SplashPath *getGlyphPath(Guint c);
 
 private:
 
