@@ -38,6 +38,7 @@ class QSignalMapper;
 class QSplitter;
 class QStackedLayout;
 class QStackedWidget;
+class QTextBrowser;
 class QTimer;
 class QToolBar;
 class QToolButton;
@@ -153,6 +154,26 @@ public:
   // Execute a command [cmd], with [event] for context.
   void execCmd(const char *cmd, QInputEvent *event);
 
+  // Used by XpdfApp::saveSession() to save session info for one
+  // window.
+  void saveSession(FILE *out, int format);
+
+  // Used by XpdfApp::loadSession() to load a session for one window.
+  void loadSession(FILE *in, int format);
+
+  // Returns true if this viewer contains a single empty tab.
+  GBool isEmpty();
+
+  // Goto [page] in the current tab.
+  void gotoPage(int page);
+
+  // Goto [destName] in the current tab.
+  void gotoNamedDestination(QString destName);
+
+public slots:
+
+  bool close();
+
 private slots:
 
   void remoteServerConnection();
@@ -179,8 +200,10 @@ private slots:
 
   void openMenuAction();
   void openInNewWinMenuAction();
+  void closeMenuAction();
   void reloadMenuAction();
   void saveAsMenuAction();
+  void loadSessionMenuAction();
   void saveImageMenuAction();
 #if XPDFWIDGET_PRINTING
   void printMenuAction();
@@ -193,12 +216,14 @@ private slots:
   void sideBySideContinuousModeMenuAction();
   void horizontalContinuousModeMenuAction();
   void fullScreenMenuAction(bool checked);
+  void reverseVideoMenuAction(bool checked);
   void rotateClockwiseMenuAction();
   void rotateCounterclockwiseMenuAction();
   void zoomToSelectionMenuAction();
   void toggleToolbarMenuAction(bool checked);
   void toggleSidebarMenuAction(bool checked);
   void viewPageLabelsMenuAction(bool checked);
+  void documentInfoMenuAction();
   void newTabMenuAction();
   void newWindowMenuAction();
   void closeTabMenuAction();
@@ -216,7 +241,7 @@ private slots:
   void forwardButtonPressed();
   void zoomOutButtonPressed();
   void zoomInButtonPressed();
-  void zoomIndexChanged(const QString &zoomText);
+  void zoomIndexChanged(int idx);
   void zoomEditingFinished();
   void fitWidthButtonPressed();
   void fitPageButtonPressed();
@@ -228,6 +253,8 @@ private slots:
   void newTabButtonPressed();
 
   void switchTab(QListWidgetItem *current, QListWidgetItem *previous);
+  void tabsReordered(const QModelIndex &srcParent, int srcStart, int srcEnd,
+		     const QModelIndex &destParent, int destRow);
   void infoComboBoxChanged(int idx);
   void outlineItemClicked(const QModelIndex& idx);
   void layerItemClicked(const QModelIndex& idx);
@@ -280,18 +307,22 @@ private:
   void cmdGotoLastPage(GString *args[], int nArgs, QInputEvent *event);
   void cmdGotoPage(GString *args[], int nArgs, QInputEvent *event);
   void cmdHelp(GString *args[], int nArgs, QInputEvent *event);
+  void cmdHideMenuBar(GString *args[], int nArgs, QInputEvent *event);
   void cmdHideToolbar(GString *args[], int nArgs, QInputEvent *event);
   void cmdHorizontalContinuousMode(GString *args[], int nArgs, QInputEvent *event);
   void cmdLinearSelectMode(GString *args[], int nArgs, QInputEvent *event);
+  void cmdLoadSession(GString *args[], int nArgs, QInputEvent *event);
   void cmdLoadTabState(GString *args[], int nArgs, QInputEvent *event);
   void cmdNewTab(GString *args[], int nArgs, QInputEvent *event);
   void cmdNewWindow(GString *args[], int nArgs, QInputEvent *event);
   void cmdNextPage(GString *args[], int nArgs, QInputEvent *event);
   void cmdNextPageNoScroll(GString *args[], int nArgs, QInputEvent *event);
   void cmdNextTab(GString *args[], int nArgs, QInputEvent *event);
+  void cmdNormalVideoMode(GString *args[], int nArgs, QInputEvent *event);
   void cmdOpen(GString *args[], int nArgs, QInputEvent *event);
   void cmdOpenErrorWindow(GString *args[], int nArgs, QInputEvent *event);
   void cmdOpenFile(GString *args[], int nArgs, QInputEvent *event);
+  void cmdOpenFile2(GString *args[], int nArgs, QInputEvent *event);
   void cmdOpenFileAtDest(GString *args[], int nArgs, QInputEvent *event);
   void cmdOpenFileAtDestIn(GString *args[], int nArgs, QInputEvent *event);
   void cmdOpenFileAtPage(GString *args[], int nArgs, QInputEvent *event);
@@ -313,11 +344,13 @@ private:
   void cmdQuit(GString *args[], int nArgs, QInputEvent *event);
   void cmdRaise(GString *args[], int nArgs, QInputEvent *event);
   void cmdReload(GString *args[], int nArgs, QInputEvent *event);
+  void cmdReverseVideoMode(GString *args[], int nArgs, QInputEvent *event);
   void cmdRotateCW(GString *args[], int nArgs, QInputEvent *event);
   void cmdRotateCCW(GString *args[], int nArgs, QInputEvent *event);
   void cmdRun(GString *args[], int nArgs, QInputEvent *event);
   void cmdSaveAs(GString *args[], int nArgs, QInputEvent *event);
   void cmdSaveImage(GString *args[], int nArgs, QInputEvent *event);
+  void cmdSaveSession(GString *args[], int nArgs, QInputEvent *event);
   void cmdSaveTabState(GString *args[], int nArgs, QInputEvent *event);
   void cmdScrollDown(GString *args[], int nArgs, QInputEvent *event);
   void cmdScrollDownNextPage(GString *args[], int nArgs, QInputEvent *event);
@@ -336,7 +369,12 @@ private:
   void cmdSelectLine(GString *args[], int nArgs, QInputEvent *event);
   void cmdSelectWord(GString *args[], int nArgs, QInputEvent *event);
   void cmdSetSelection(GString *args[], int nArgs, QInputEvent *event);
+  void cmdShowAttachmentsPane(GString *args[], int nArgs, QInputEvent *event);
+  void cmdShowDocumentInfo(GString *args[], int nArgs, QInputEvent *event);
   void cmdShowKeyBindings(GString *args[], int nArgs, QInputEvent *event);
+  void cmdShowLayersPane(GString *args[], int nArgs, QInputEvent *event);
+  void cmdShowMenuBar(GString *args[], int nArgs, QInputEvent *event);
+  void cmdShowOutlinePane(GString *args[], int nArgs, QInputEvent *event);
   void cmdShowToolbar(GString *args[], int nArgs, QInputEvent *event);
   void cmdShrinkSidebar(GString *args[], int nArgs, QInputEvent *event);
   void cmdSideBySideContinuousMode(GString *args[], int nArgs, QInputEvent *event);
@@ -347,6 +385,8 @@ private:
   void cmdStartSelection(GString *args[], int nArgs, QInputEvent *event);
   void cmdToggleContinuousMode(GString *args[], int nArgs, QInputEvent *event);
   void cmdToggleFullScreenMode(GString *args[], int nArgs, QInputEvent *event);
+  void cmdToggleMenuBar(GString *args[], int nArgs, QInputEvent *event);
+  void cmdToggleReverseVideoMode(GString *args[], int nArgs, QInputEvent *event);
   void cmdToggleSelectMode(GString *args[], int nArgs, QInputEvent *event);
   void cmdToggleSidebar(GString *args[], int nArgs, QInputEvent *event);
   void cmdToggleSidebarMoveResizeWin(GString *args[], int nArgs, QInputEvent *event);
@@ -361,6 +401,7 @@ private:
   void cmdZoomOut(GString *args[], int nArgs, QInputEvent *event);
   void cmdZoomPercent(GString *args[], int nArgs, QInputEvent *event);
   void cmdZoomToSelection(GString *args[], int nArgs, QInputEvent *event);
+  int getFindCaseFlag();
   int scaleScroll(int delta);
   void followLink(QInputEvent *event, GBool onlyIfNoSel,
 		  GBool newTab, GBool newWindow);
@@ -408,6 +449,10 @@ private:
   void statusIndicatorOk();
   void statusIndicatorError();
   void showFindError();
+  void createDocumentInfoDialog();
+  void updateDocumentInfoDialog(XpdfWidget *view);
+  QString createDocumentInfoMetadataHTML(XpdfWidget *view);
+  QString createDocumentInfoFontsHTML(XpdfWidget *view);
   void createKeyBindingsDialog();
   QString createKeyBindingsHTML();
   void createAboutDialog();
@@ -421,6 +466,7 @@ private:
   QMenuBar *mainMenu;
   QMenu *displayModeSubmenu;
   QAction *fullScreenMenuItem;
+  QAction *reverseVideoMenuItem;
   QAction *toggleToolbarMenuItem;
   QAction *toggleSidebarMenuItem;
   QAction *viewPageLabelsMenuItem;
@@ -442,7 +488,9 @@ private:
   QList<QVariant> indicatorIcons;
   QList<QVariant> indicatorErrIcons;
   QLineEdit *findEdit;
+  QAction *findCaseInsensitiveAction;
   QAction *findCaseSensitiveAction;
+  QAction *findSmartCaseAction;
   QAction *findWholeWordsAction;
 
   // sidebar pane
@@ -461,6 +509,9 @@ private:
 
   GList *tabInfo;		// [XpdfTabInfo]
   XpdfTabInfo *currentTab;
+  XpdfTabInfo *lastOpenedTab;
+
+  bool reverseVideo;
 
   double scaleFactor;
 
@@ -470,6 +521,9 @@ private:
   QTimer *findErrorTimer;
 
   XpdfErrorWindow *errorWindow;
+  QDialog *documentInfoDialog;
+  QTextBrowser *documentInfoMetadataTab;
+  QTextBrowser *documentInfoFontsTab;
   QDialog *keyBindingsDialog;
   QDialog *aboutDialog;
 #if XPDFWIDGET_PRINTING
