@@ -22,6 +22,7 @@ class GfxDeviceNColorSpace;
 class GfxSeparationColorSpace;
 class GfxShading;
 class GfxState;
+class LocalParams;
 
 //------------------------------------------------------------------------
 // GfxBlendMode
@@ -1128,7 +1129,8 @@ public:
   // Construct a default GfxState, for a device with resolution <hDPI>
   // x <vDPI>, page box <pageBox>, page rotation <rotateA>, and
   // coordinate system specified by <upsideDown>.
-  GfxState(double hDPIA, double vDPIA, PDFRectangle *pageBox,
+  GfxState(LocalParams *localParamsA,
+	   double hDPIA, double vDPIA, PDFRectangle *pageBox,
 	   int rotateA, GBool upsideDown
 	   );
 
@@ -1184,6 +1186,7 @@ public:
   int getLineCap() { return lineCap; }
   double getMiterLimit() { return miterLimit; }
   GBool getStrokeAdjust() { return strokeAdjust; }
+  GBool getAlphaIsShape() { return alphaIsShape; }
   GfxFont *getFont() { return font; }
   double getFontSize() { return fontSize; }
   double *getTextMat() { return textMat; }
@@ -1245,7 +1248,7 @@ public:
   void setFillOverprint(GBool op) { fillOverprint = op; }
   void setStrokeOverprint(GBool op) { strokeOverprint = op; }
   void setOverprintMode(int opm) { overprintMode = opm; }
-  void setRenderingIntent(GfxRenderingIntent ri) { renderingIntent = ri; }
+  void setRenderingIntent(GfxRenderingIntent ri);
   void setTransfer(Function **funcs);
   void setLineWidth(double width) { lineWidth = width; }
   void setLineDash(double *dash, int length, double start);
@@ -1254,6 +1257,7 @@ public:
   void setLineCap(int lineCap1) { lineCap = lineCap1; }
   void setMiterLimit(double limit) { miterLimit = limit; }
   void setStrokeAdjust(GBool sa) { strokeAdjust = sa; }
+  void setAlphaIsShape(GBool ais) { alphaIsShape = ais; }
   void setFont(GfxFont *fontA, double fontSizeA)
     { font = fontA; fontSize = fontSizeA; }
   void setTextMat(double a, double b, double c,
@@ -1313,6 +1317,8 @@ public:
 
 private:
 
+  LocalParams *localParams;
+
   double hDPI, vDPI;		// resolution
   double ctm[6];		// coord transform matrix
   double px1, py1, px2, py2;	// page corners (user coords)
@@ -1346,6 +1352,7 @@ private:
   int lineCap;			// line cap style
   double miterLimit;		// line miter limit
   GBool strokeAdjust;		// stroke adjustment
+  GBool alphaIsShape;
 
   GfxFont *font;		// font
   double fontSize;		// font size
