@@ -8,10 +8,6 @@
 
 #include <aconf.h>
 
-#ifdef USE_GCC_PRAGMAS
-#pragma implementation
-#endif
-
 #include "GString.h"
 #include "GHash.h"
 #include "Object.h"
@@ -46,9 +42,13 @@
 //------------------------------------------------------------------------
 
 XFAFieldLayoutInfo::XFAFieldLayoutInfo(XFAFieldLayoutHAlign hAlignA,
-				       XFAFieldLayoutVAlign vAlignA) {
+				       XFAFieldLayoutVAlign vAlignA,
+				       double marginLeftA,
+				       double marginRightA) {
   hAlign = hAlignA;
   vAlign = vAlignA;
+  marginLeft = marginLeftA;
+  marginRight = marginRightA;
 }
 
 //------------------------------------------------------------------------
@@ -293,8 +293,10 @@ void XFAScanner::scanNode(ZxElement *elem,
   }
 
   if (elem->isElement("field")) {
-    scanField(elem, childName, childFullName, exclGroupName,
-	      dataElem, formValues);
+    if (childName && childFullName) {
+      scanField(elem, childName, childFullName, exclGroupName,
+		dataElem, formValues);
+    }
   } else {
     GString *childExclGroupName;
     if (elem->isElement("exclGroup")) {
@@ -485,7 +487,16 @@ XFAFieldLayoutInfo *XFAScanner::getFieldLayoutInfo(ZxElement *elem) {
       vAlign = xfaFieldLayoutVAlignBottom;
     }
   }
-  return new XFAFieldLayoutInfo(hAlign, vAlign);
+  double marginLeft = 0;
+  double marginRight = 0;
+  ZxAttr *marginAttr;
+  if ((marginAttr = paraElem->findAttr("marginLeft"))) {
+    marginLeft = getMeasurement(marginAttr->getValue());
+  }
+  if ((marginAttr = paraElem->findAttr("marginRight"))) {
+    marginRight = getMeasurement(marginAttr->getValue());
+  }
+  return new XFAFieldLayoutInfo(hAlign, vAlign, marginLeft, marginRight);
 }
 
 XFAFieldPictureInfo *XFAScanner::getFieldPictureInfo(ZxElement *elem) {
